#=====================================================================
#   MTYPE.pm: guess media type
#
#   Copyright(c) 2001, Nobuchika Oishi (BSC CONSULTING).
#
#   e-mail : bigstone@my.email.ne.jp
#   support: http://www.din.or.jp/~bigstone/cgilab/index.html
#
#   THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND
#   ANY EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED.
#
#   ver 1.0.2 : updated last on 2001/12/12
#
#=====================================================================
package MTYPE;

use strict;
use integer;
use vars qw($VERSION %SUFFIX_TYPE %SUFFIX_EXT %SUFFIX_ENCODE);

$VERSION = '1.0.2';

%SUFFIX_TYPE = (
    'txt'   => 'text/plain',
    'html'  => 'text/html',
    'htm'   => 'text/html',
    'gif'   => 'image/gif',
    'jpg'   => 'image/jpeg',
    'jpeg'  => 'image/jpeg',
    'png'   => 'image/png',
    'tiff'  => 'image/tiff',
    'tif'   => 'image/tiff',
    'bmp'   => 'image/bmp',
);

%SUFFIX_ENCODE = (
    'Z'   => 'compress',
    'gz'  => 'gzip',
    'hqx' => 'x-hqx',
    'uu'  => 'x-uuencode',
    'z'   => 'x-pack',
    'bz2' => 'x-bzip2',
);

%SUFFIX_EXT = (
    'text/plain' => 'txt',
    'text/html'  => 'html',
    'image/gif'  => 'gif',
    'image/jpeg' => 'jpg',
    'image/png'  => 'png',
    'image/tiff' => 'tiff',
    'image/bmp'  => 'bmp',
);

read_media_types();

sub guess_type {
    my $file = @_ > 1 ? $_[1] : shift;
    return undef unless defined $file;
    my($suffix, $ctype, $encode);
    if(($suffix) = $file =~ m!([^./:]+)$!o){
        $ctype  = $SUFFIX_TYPE{$suffix}   || $SUFFIX_TYPE{lc($suffix)};
        $encode = $SUFFIX_ENCODE{$suffix} || $SUFFIX_ENCODE{lc($suffix)};
    }
    $ctype ||= 'application/octet-stream';
    return wantarray ? ($ctype, $encode) : $ctype;
}

sub media_suffix {
    my $type = @_ > 1 ? $_[1] : shift;
    my $ext  = $SUFFIX_EXT{$type} || $SUFFIX_EXT{lc($type)};
    unless($ext){
        if($type =~ /^(image|text)/o){
            ($ext) = $type =~ /(\w+)$/o;
        }
    }
    $ext;
}

sub add_type {
    my($type, @exts) = @_;
    foreach my $ext (@exts){
        $ext =~ s/^\.//o;
        $SUFFIX_TYPE{$ext} = $type;
    }
    $SUFFIX_EXT{$type} = $exts[0] if @exts;
}

sub add_encode {
    my($type, @exts) = @_;
    foreach my $ext (@exts){
        $ext =~ s/^\.//o;
        $SUFFIX_ENCODE{$ext} = $type;
    }
}

sub read_media_types {
    my(@files) = @_;
    unless(@files){
        push(@files, "./media.types") if(-f "./media.types");
    }
    unless(@files){
        @files = map {"$_/media.types"} @INC;
    }
    my($file, $type, @exts);
    local($/) = "\n";
    local(*FH);
    foreach $file (@files){
        open(FH, "< $file") or next;
        while (<FH>){
            next if /^\s*#/o;
            next if /^\s*$/o;
            s/#.*//o;
            ($type, @exts) = split(/[\s\t]+/o);
            add_type($type, @exts);
        }
        close(FH);
    }
}

1;

__END__
